-- hakto_utils.lua
-- Last Change Date: 26.02.17

-- Some additional changes for this awesome scenario

-- Changes from Oarc Default (in the files):
-- General - Display Settings, custom hostname display
-- Seperate Spawns GUI - Messages and display style
-- Oarc Utils - Autofill oven, Resource generation code cleanup
-- Tag - Changed some Tags

-- Additional Modifications: 
-- Player List, Faster Movement, Faster and higher cargosize Bots, Rules GUI
-- Hostmessage customization, Kickstart Item Package

server_settings_style = {
    minimal_width = 450,
    maximal_height = 14,
    font = "default-small-semibold",
    font_color = {r=40,g=180,b=255}
}

server_settings_style_s = {
    minimal_width = 450,
    maximal_height = 14,
    font = "default-small-semibold",
    font_color = {r=40,g=180,b=255}
}

message_style_s = {
    minimal_width = 450,
    maximal_height = 14,
    font = "default-small-semibold"
}

-- Misc

-- Notify Admin Event
function NotifyAdmin(message)
    for _, player in pairs(game.players) do 
        if player.admin then
            player.print(message)
        end
    end
end

function ConvertHours (tick)
    local hour = tostring(math.floor(tick * (1 /(60*game.speed)) / 3600))
    return hour
end

function ConvertMinutes (tick)
    local minutes = math.floor((tick * (1 /(60*game.speed))) / 60)
    return minutes
end

function SetHostServerMessages(event)
    local player = game.players[event.player_index]
    
    if global.serverHost[1] and global.serverHost[1] ~= nil then
        if global.serverHost[1] == "Hakto" then
            global.welcome_msg = "Welcome to Hakto's Server!"
            global.welcome_msg_title = "Welcome to Hakto's Server!"
            MODIFIED_INFO1 = MODIFIED_INFO1_HAKTO
            MODIFIED_INFO2 = MODIFIED_INFO2_HAKTO
            GAME_NAME = GAME_NAME_HAKTO
        else
            global.welcome_msg = WELCOME_MSG_HEAD .. global.serverHost[1] .. WELCOME_MSG_TAIL
            global.welcome_msg_title = WELCOME_MSG_HEAD .. global.serverHost[1] .. WELCOME_MSG_TAIL
        end
    end
end

-- Workaround to identify the host for custom messages
-- somehow players[1] is empty you dont stay long enough after the init join
function CreateHostTable()
    if (global.serverHost == nil) then
        global.serverHost = {}
    end
end

function LogHost(player)
    if (global.serverHost[1] == nil) then        
        table.insert(global.serverHost, player.name)
    end
end

-- Allow player to have an larger inventory
function GivePlayerLargerInventory(player)
    if player ~= nil then
        player.character.character_inventory_slots_bonus = INVENTORY_BONUS
    end
end

-- Raise the movementspeed of the player
function GivePlayerFasterMovement(player)
    if player ~= nil then
        player.character.character_running_speed_modifier = MOVESPEED_BONUS
    end
end

-- Kickstart
function EnableAutomatedConstruction(force)
    force.technologies['automated-construction'].researched=true;
end

function EnableEfficientRobots(force)
    force.worker_robots_storage_bonus = EFFICIENT_BOTS_CAPACITYBONUS;
    force.worker_robots_speed_modifier = EFFICIENT_BOTS_SPEEDBONUS;
end

-- Give player these default items.
function GivePlayerItems(player)
    player.insert{name="steel-axe", count = 5}
    player.insert{name="pistol", count=1}
    player.insert{name="firearm-magazine", count=100}
end

-- Additional Kickstart items
function GivePlayerStarterItems(player)
    GivePlayerItems(player)
    if ENABLE_KICKSTART then
        player.insert{name="steel-axe", count = 5}
        player.insert{name="lab", count=1}
        player.insert{name="science-pack-1", count=10}
        player.insert{name="burner-mining-drill", count = 10}
        player.insert{name="stone-furnace", count = 10}
        player.insert{name="power-armor", count = 1}
        player.insert{name="fusion-reactor-equipment", count = 1}
        player.insert{name="personal-roboport-equipment", count = 2}
        player.insert{name="construction-robot", count = 20}
        player.insert{name="battery-mk2-equipment", count = 1}
        player.insert{name="blueprint", count = 3}
        player.insert{name="deconstruction-planner", count = 1}
    else
        player.insert{name="burner-mining-drill", count = 3}
        player.insert{name="stone-furnace", count = 1}
    end
end

-- Playerlisting

playerlist_host = {
    minimal_width = 160,
    maximal_width = 160,
    maximal_height = 16,
    font_color = {r=255,g=0,b=0}
}

playerlist_admin = {
    minimal_width = 160,
    maximal_width = 160,
    maximal_height = 16,
    font_color = {r=255,g=60,b=0}
}

playerlist_8h = {
    minimal_width = 160,
    maximal_width = 160,
    maximal_height = 16,
    font_color = {r=255,g=140,b=0}
}

playerlist_4h = {
    minimal_width = 160,
    maximal_width = 160,
    maximal_height = 16,
    font_color = {r=255,g=180,b=0}
}

playerlist_2h = {
    minimal_width = 160,
    maximal_width = 160,
    maximal_height = 16,
    font_color = {r=255,g=220,b=0}
}

playerlist_0h = {
    minimal_width = 160,
    maximal_width = 160,
    maximal_height = 16,
    font_color = {r=255,g=255,b=255}
}

playerlist_spacer = {
    minimal_width = 160,
    maximal_width = 160,
    minimal_height = 5,
    maximal_height = 5,
    font_color = {r=0,g=0,b=0}
}

function CreatePlayerListGui(event)
    local player = game.players[event.player_index]
    if player.gui.top.playerlistbutton == nil then
        player.gui.top.add{name="playerlistbutton", type="button", caption="Players", tooltip= "Display Active Players"}
    end   
end

function PlayerListGuiClick(event) 
    if not (event and event.element and event.element.valid) then return end
    local player = game.players[event.element.player_index]
    local name = event.element.name
    
    if (name == "playerlistbutton") then
        local frame = player.gui.left["PlayerListFrame"]
        if (frame) then
            frame.destroy()
        else
            player.gui.left.add{name= "PlayerListFrame", type = "frame", direction = "vertical", caption = "Active Players"}
            local playerListGui = player.gui.left.PlayerListFrame
            
            for key, player in pairs(game.connected_players) do
                if player ~= nil and player.valid then
                    
                    local PTMinutes = ConvertMinutes(player.online_time)
                    local PTHours = ConvertHours(player.online_time)
                    
                    if player.admin == true then
                        -- global.serverHost[1] will be the name of the save creator
                        if (global.serverHost[1] == player.name) then
                            playerListGui.add{type = "label",  name = player.name, style="caption_label_style", caption={"", player.name, " (", PTHours, "h, Host)"}}
                            ApplyStyle(playerListGui[player.name], playerlist_host)
                        else
                            playerListGui.add{type = "label",  name = player.name, style="caption_label_style", caption={"", player.name, " (", PTHours, "h, Admin)"}}
                            ApplyStyle(playerListGui[player.name], playerlist_admin)
                        end
                    else
                        playerListGui.add{type = "label",  name = player.name, style="caption_label_style", caption={"", player.name, " (", PTHours, "h)",}}
                        
                        if PTMinutes >= 480 then
                            ApplyStyle(playerListGui[player.name], playerlist_8h)
                        elseif PTMinutes >= 240 then
                            ApplyStyle(playerListGui[player.name], playerlist_4h)
                        elseif PTMinutes >= 120 then
                            ApplyStyle(playerListGui[player.name], playerlist_2h)
                        else
                            ApplyStyle(playerListGui[player.name], playerlist_0h)
                        end
                    end
                end
            end
            
            playerListGui.add{name = "guispacer", type = "label", caption=" "}
            ApplyStyle(playerListGui.guispacer, playerlist_spacer)
        end
    end    
end

-- Server Rules

function CreateRuleGui(event)
    local player = game.players[event.player_index]
    if player.gui.top.serverrule_btn == nil then
        player.gui.top.add{name="serverrule_btn", type="button", caption="Info", tooltip= "Rules & Infos"}
    end   
end

rules_label_style = {
    minimal_width = 400,
    maximal_width = 400,
    maximal_height = 12,
    font_color = {r=1,g=1,b=1}
}

function RuleGuiClick(event) 
    if not (event and event.element and event.element.valid) then return end
    local player = game.players[event.element.player_index]
    local name = event.element.name
    
    if (name == "serverrule_btn") then
        local frame = player.gui.center["ruleframe"]
        if (frame) then
            frame.destroy()
        else
            player.gui.center.add{name = "ruleframe",type = "frame",direction = "vertical",caption=RULEHEADLINE}
            
            local ruleGui = player.gui.center.ruleframe
            ruleGui.add{name = "rule1", type = "label", caption=RULE1}
            ruleGui.add{name = "rule2", type = "label", caption=RULE2}
            ruleGui.add{name = "rule3", type = "label", caption=RULE3}
            ruleGui.add{name = "rule4", type = "label", caption=RULE4}
            ruleGui.add{name = "rule5", type = "label", caption=RULE5}
            ruleGui.add{name = "rule6", type = "label", caption=RULE6}
            ruleGui.add{name = "rule7", type = "label", caption=RULE7}
            
            ApplyStyle(ruleGui.rule1, rules_label_style)
            ApplyStyle(ruleGui.rule2, rules_label_style)
            ApplyStyle(ruleGui.rule3, rules_label_style)
            ApplyStyle(ruleGui.rule4, rules_label_style)
            ApplyStyle(ruleGui.rule5, rules_label_style)
            ApplyStyle(ruleGui.rule6, rules_label_style)
            ApplyStyle(ruleGui.rule1, rules_label_style)
            
            ruleGui.add{name = "rulespacer1", type = "label", caption=" "}            
            ruleGui.add{name = "rulegamename", type = "label", caption= GAME_NAME }            
            ruleGui.add{name = "rulegamediscord", type = "label", caption= MODIFIED_INFO1 }            
            ruleGui.add{name = "rulespacer2", type = "label", caption=" "}
            
            ApplyStyle(ruleGui.rulegamename, rules_label_style)
            ApplyStyle(ruleGui.rulegamediscord, rules_label_style)
            ApplyStyle(ruleGui.rulespacer1, my_spacer_style)
            ApplyStyle(ruleGui.rulespacer2, my_spacer_style)
        end
    end    
end